
package mobicomp.net;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

import mobicomp.interfaces.MHMessage;

/**
 * TODO
 * 
 * @author Andreas Wetzel <andi@spamato.net>
 */
public class MHMessageImpl implements MHMessage
{
    private byte type;
    private int floodId = 0;
    private int sender;
    private int receiver;
    private int ttl;
    private int routeIndex;
    private Vector route = new Vector();
    private byte[] data;
    
    public MHMessageImpl() {
    }
    
    public MHMessageImpl( byte[] data) throws IOException {
        ByteArrayInputStream bis = new ByteArrayInputStream( data);
        DataInputStream in = new DataInputStream( bis);
        
        type = in.readByte();
        floodId = in.readUnsignedShort();
        sender = in.readUnsignedShort();
        receiver = in.readUnsignedShort();
        ttl = in.readUnsignedByte();
        int routelen = in.readUnsignedByte();
        route = new Vector( routelen);
        for( int i=0; i<routelen; i++) {
            route.add( new Integer( in.readUnsignedShort()));
        }
        
    }
    
    /**
     * @see mobicomp.interfaces.MHMessage#setType(byte)
     */
    public void setType( byte b) {
        type = b;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getType()
     */
    public byte getType() {
        return type;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setFloodId(int)
     */
    public void setFloodId( int id) {
        floodId = id;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getFloodId()
     */
    public int getFloodId() {
        return floodId;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setSender(int)
     */
    public void setSender( int addr) {
        sender = addr;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getSender()
     */
    public int getSender() {
        return sender;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setReceiver(int)
     */
    public void setReceiver( int addr) {
        receiver = addr;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getReceiver()
     */
    public int getReceiver() {
        return receiver;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setTTL(int)
     */
    public void setTTL( int ttl) {
        this.ttl = ttl;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getTTL()
     */
    public int getTTL() {
        return ttl;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setRouteIndex(int)
     */
    public void setRouteIndex( int routeIndex) {
        this.routeIndex = routeIndex;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getRouteIndex()
     */
    public int getRouteIndex() {
        return routeIndex;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setRoute(java.util.Vector)
     */
    public void setRoute( Vector route) {
        this.route = route;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getRoute()
     */
    public Vector getRoute() {
        return route;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#setData(byte[])
     */
    public void setData( byte[] data) {
        this.data = data;
    }

    /**
     * @see mobicomp.interfaces.MHMessage#getData()
     */
    public byte[] getData() {
        return data;
    }
    
    public byte[] toByteArray() {
        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            DataOutputStream out = new DataOutputStream( bos);

            out.writeByte( type);
            out.writeShort( floodId);
            out.writeShort( sender);
            out.writeShort( receiver);
            out.writeByte( ttl);
            out.writeByte( route.size());
            for( int i=0; i<route.size(); i++) {
                Integer ii = (Integer)route.get( i);
                int hop = ii.intValue();
                out.writeShort( hop);
            }
            
            out.close();
            
            return bos.toByteArray();
        }
        catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
}
